//
//  EntityManager.swift
//  MonsterWars
//
//  Created by Main Account on 11/3/15.
//  Copyright © 2015 Razeware LLC. All rights reserved.
//

import Foundation
import SpriteKit
import GameplayKit

class EntityManager {

  // 1
  var entities = Set<GKEntity>()
  let scene: SKScene
  var toRemove = Set<GKEntity>()
  
  lazy var componentSystems: [GKComponentSystem] = {
    let castleSystem = GKComponentSystem(componentClass: CastleComponent.self)
    let moveSystem = GKComponentSystem(componentClass: MoveComponent.self)
    return [castleSystem, moveSystem]
  }()
  
  // 2
  init(scene: SKScene) {
    self.scene = scene
  }
  
  // 3
  func add(entity: GKEntity) {
    entities.insert(entity)
    
    if let spriteNode = entity.componentForClass(SpriteComponent.self)?.node {
      scene.addChild(spriteNode)
    }

    for componentSystem in componentSystems {
      componentSystem.addComponentWithEntity(entity)
    }
  }
  
  // 4
  func remove(entity: GKEntity) {
    if let spriteNode = entity.componentForClass(SpriteComponent.self)?.node {
      spriteNode.removeFromParent()
    }
    
    entities.remove(entity)
    toRemove.insert(entity)
  }
  
  func update(deltaTime: CFTimeInterval) {
    for componentSystem in componentSystems {
      componentSystem.updateWithDeltaTime(deltaTime)
    }
    
    for curRemove in toRemove {
      for componentSystem in componentSystems {
        componentSystem.removeComponentWithEntity(curRemove)
      }
    }
    toRemove.removeAll()
  }
  
  func castleForTeam(team: Team) -> GKEntity? {
    for entity in entities {
      if let teamComponent = entity.componentForClass(TeamComponent.self),
             _ = entity.componentForClass(CastleComponent.self) {
        if teamComponent.team == team {
          return entity
        }
      }
    }
    return nil
  }
  
  func spawnQuirk(team: Team) {
    // 1
    guard let teamEntity = castleForTeam(team),
          teamCastleComponent = teamEntity.componentForClass(CastleComponent.self),
          teamSpriteComponent = teamEntity.componentForClass(SpriteComponent.self) else {
      return
    }
    
    // 2
    if teamCastleComponent.coins < costQuirk {
      return
    }
    teamCastleComponent.coins -= costQuirk
    scene.runAction(SoundManager.sharedInstance.soundSpawn)
    
    // 3
    let monster = Quirk(team: team, entityManager: self)
    if let spriteComponent = monster.componentForClass(SpriteComponent.self) {
      spriteComponent.node.position = CGPoint(x: teamSpriteComponent.node.position.x, y: CGFloat.random(min: scene.size.height * 0.25, max: scene.size.height * 0.75))
      spriteComponent.node.zPosition = 2
    }
    add(monster)
  }
  
  func entitiesForTeam(team: Team) -> [GKEntity] {
    
    return entities.flatMap{ entity in
      if let teamComponent = entity.componentForClass(TeamComponent.self) {
        if teamComponent.team == team {
          return entity
        }
      }
      return nil
    }
    
  }

  func moveComponentsForTeam(team: Team) -> [MoveComponent] {
    let entities = entitiesForTeam(team)
    var moveComponents = [MoveComponent]()
    for entity in entities {
      if let moveComponent = entity.componentForClass(MoveComponent.self) {
        moveComponents.append(moveComponent)
      }
    }
    return moveComponents
  }
  
}